function [] = sdolab_gw (n,x,s);

%%
% [] = sdolab_gw (n,x,s);
%
% Computes the matrice G and W as global variables SDO_G and SDO_W.
%
% Version 0.4, March, 2017.

%-----------------------------------------------------------------------
%
% Author: Jean Charles Gilbert, INRIA.
% Copyright 2017, INRIA.
%
% SDOlab is distributed under the terms of the Q Public License version
% 1.0.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Q Public
% License version 1.0 for more details.
%
% You should have received a copy of the Q Public License version 1.0
% along with this program.  If not, see
% <http://doc.trolltech.com/3.0/license.html>.
%
%-----------------------------------------------------------------------

  global SDO_info SDO_options SDO_values
  global SDO_RX SDO_RX_piv SDO_RS SDO_RS_piv SDO_D SDO_V SDO_sqrtd SDO_G SDO_W SDO_mub

% Factorize X

  x(1:n+1:end) = real(diag(mat(x)));			% remove imaginary scorias on the diagonal of X (due to rounding error)

  if SDO_options.cholesky == SDO_values.cholesky_chol
    [SDO_RX,pos] = chol(mat(x));			% upper triangular Cholesky factor of X
    if pos	% X is not positive definite
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_gw: failure in the Cholesky factorization of X (lack of positive definiteness after %i steps on %i)',pos,n);
      end
      return
    end
  else
    [SDO_RX,SDO_RX_piv,fail] = zchdc(mat(x),true);	% upper triangular Cholesky factor of X
    if fail	% X is not positive definite
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_gw: failure in the Cholesky factorization of X (lack of positive definiteness after %i steps on %i)',fail,n);
      end
      return
    end
  end

% Factorize S

  s(1:n+1:end) = real(diag(mat(s)));			% remove imaginary scorias on the diagonal of S (due to rounding error)

  if SDO_options.cholesky == SDO_values.cholesky_chol
    [SDO_RS,pos] = chol(mat(s));			% upper triangular Cholesky factor of S
    if pos	% S is not positive definite
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_gw: failure in the Cholesky factorization of S (lack of positive definiteness after %i steps on %i)',pos,n);
      end
      return
    end
  else
    [SDO_RS,SDO_RS_piv,fail] = zchdc(mat(s),true);	% upper triangular Cholesky factor of S
    if fail	% S is not positive definite
      SDO_info.flag = SDO_values.fail_on_chol;
      if SDO_options.verb >= 2
        fprintf(SDO_options.fout,'\nsdolab_gw: failure in the Cholesky factorization of S (lack of positive definiteness after %i steps on %i)',fail,n);
      end
      return
    end
  end

% Compute G and W

  if SDO_options.cholesky == SDO_values.cholesky_chol
    [U,SDO_D,SDO_V] = svd(SDO_RS*SDO_RX');
    SDO_sqrtd = sqrt(diag(SDO_D));
    SDO_G = SDO_RX'*(SDO_V*diag(1./SDO_sqrtd));
    SDO_W = SDO_G*SDO_G';
  else
    PX = sparse(double(SDO_RX_piv),[1:n],ones(n,1));
    PS = sparse(double(SDO_RS_piv),[1:n],ones(n,1));
    [U,SDO_D,SDO_V] = svd(SDO_RS*(PS'*PX)*SDO_RX');
    SDO_sqrtd = sqrt(diag(SDO_D));
    SDO_G = PX*(SDO_RX'*(SDO_V*diag(1./SDO_sqrtd)));
    SDO_W = SDO_G*SDO_G';
  end

% Compute mub

  SDO_mub = real((x'*s)/n);				% remove imaginary scorias

% Return

  return

end
