% Tune Matlab

  clear all;

  format short
% format long
% format compact

% Choose the output channel

  fout = fopen('res','w');	% on the file named 'res'
% fout = 1;			% on the screen

% Date

  fprintf(fout,'(%s)\n',datestr(now));

% Set the number of cores

  warning('off','MATLAB:maxNumCompThreads:Deprecated');
  ncores = 1;			% set the number of cores here (1 or 4)
  maxNumCompThreads(ncores);	% default is 4 on the MP-41212 (4 cores); restore 'maxNumCompThreads(4)' after a change!
  fprintf(fout,'\nNumber of cores: %i\n',ncores);

% Modify the Matlab search path

  addpath ../src
  addpath ../sedumi_1_3

% Select the solvers (by true/false)

  solver.sdolab04_in_complex     = true;	% Sdolab on the problem in complex numbers
  solver.sdolab04_in_complex_hc  = true;	% Sdolab on the problem with only Hermitian constraints
  solver.sdolab04_in_real        = true;	% Sdolab on the problem in real numbers
  solver.sedumi                  = true;	% SeDuMi on the problem in real numbers

% Sdolab options (can be modified according to each testcase)

  options                = [];
  options.eps            = 1.e-09;	% 1.e-10 for caseWB2, 1.e-09 for caseWB5 and case9
  options.iter           = 100;
  options.max_bigm_loop  = 10;
  options.cholesky       = 'chdc';	% 'chol' (Matlab), 'chdc' (Linpack)
  options.max_stepsize   = 0.999;	% 0.9 for case9, 0.99 for the other cases (with 'chol')
  options.neighborhood   = 1/3;		% 0.01;
  options.fout           = fout;
  options.verb           = 2;

% Test problems for the paper "Plea..." by Jean Charles Gilbert and Cédric Josz
%
  [A,b,c,x0,y0] = mmnc(10,0,100,100,fout);
% [A,b,c,x0,y0] = mmnc(20,0,100,100,fout);
% [A,b,c,x0,y0] = mmnc(50,0,100,100,fout);
% [A,b,c,x0,y0] = mmnc(100,0,100,100,fout);
% [A,b,c,x0,y0] = mmnc(200,0,100,100,fout);
% [A,b,c,x0,y0] = mmnc(500,0,100,100,fout);
%
% [A,b,c,x0,y0] = mmnc(0,10,100,100,fout);
% [A,b,c,x0,y0] = mmnc(0,20,100,100,fout);
% [A,b,c,x0,y0] = mmnc(0,50,100,100,fout);
% [A,b,c,x0,y0] = mmnc(0,100,100,100,fout);
% [A,b,c,x0,y0] = mmnc(0,200,100,100,fout);
% [A,b,c,x0,y0] = mmnc(0,500,100,100,fout);
%
% [A,b,c,x0,y0] = matpower_qcqpc('caseWB2',fout); options.max_stepsize = 0.99;
% [A,b,c,x0,y0] = matpower_qcqpc('caseWB5',fout);
% [A,b,c,x0,y0] = matpower_qcqpc('case9',fout); options.eps = 1.e-8; options.max_stepsize = 0.95;

% x0 = []; y0 = [];

  if isempty(A); return; end

  [mr,n2] = size(A.r);
  if isfield(A,'c')
    mc    = size(A.c,1);
  else
    mc    = 0;
  end
  n       = sqrt(n2);
  nn      = 2*n;

%-----------------------------------------------------------------------
% Run sdolab v-0.4 on the complex number problem
%-----------------------------------------------------------------------

  warning('off','MATLAB:nearlySingularMatrix')

  if solver.sdolab04_in_complex

    fprintf(fout,'\n*** Problem in complex numbers *********************************************\n');

    [x,y,s,infos] = sdolab(A,b,c,x0,y0,options);

    if infos.flag == 0

      X = mat(x);
      S = mat(s);
      yv = [y.r;real(y.c);imag(y.c)];
      tot = infos.timer(6);
      tpi = tot/infos.iter;

      fprintf(fout,'\nPrimal variable X (i,j,real(val),imag(val))');
      [i,j,val] = find(sparse(mat(x)));
      threshold = 1.e2*eps*norm(val);
      if threshold == 0; threshold = 1.e2*eps; end
      for k = 1:length(i)
        if norm(val(k)) > threshold
          if abs(imag(val(k))) < 1.e5*eps*abs(real(val(k)))
            fprintf(fout,'\n%2i  %2i  %22.15e',i(k),j(k),val(k));
          else
            fprintf(fout,'\n%2i  %2i  %22.15e  %22.15e',i(k),j(k),real(val(k)),imag(val(k)));
          end
        end
      end

      if mr
        fprintf(fout,'\n\nDual variable y.r (i,val)');
        [i,j,val] = find(sparse(y.r));
        threshold = 1.e2*eps*norm(val);
        if threshold == 0; threshold = 1.e2*eps; end
        for k = 1:length(i)
          if norm(val(k)) > threshold
            fprintf(fout,'\n%2i  %22.15e',i(k),val(k));
          end
        end
      end

      if mc
        fprintf(fout,'\n\nDual variable y.c (i,real(val),imag(val))');
        [i,j,val] = find(sparse(y.c));
        threshold = 1.e2*eps*norm(val);
        if threshold == 0; threshold = 1.e2*eps; end
        for k = 1:length(i)
          if norm(val(k)) > threshold
            fprintf(fout,'\n%2i  %22.15e  %22.15e',i(k),real(val(k)),imag(val(k)));
          end
        end
      end

      fprintf(fout,'\n\nDual variable S (i,j,real(val(k)),imag(val(k)))');
      [i,j,val] = find(sparse(mat(s)));
      threshold = 1.e2*eps*norm(val);
      if threshold == 0; threshold = 1.e2*eps; end
      for k = 1:length(i)
        if norm(val(k)) > threshold
          if abs(imag(val(k))) < 1.e5*eps*abs(real(val(k)))
            fprintf(fout,'\n%2i  %2i  %22.15e',i(k),j(k),val(k));
          else
            fprintf(fout,'\n%2i  %2i  %22.15e  %22.15e',i(k),j(k),real(val(k)),imag(val(k)));
          end
        end
      end

    else

      X = zeros(n);
      S = zeros(n);
      yv = zeros(mr+2*mc,1);

    end

  end

  fprintf(fout,'\n');

%-----------------------------------------------------------------------
% Run sdolab v-0.4 on the complex number problem with "Hermitian constraints"
%-----------------------------------------------------------------------

  if solver.sdolab04_in_complex_hc

    fprintf(fout,'\n*** Problem in complex numbers and Hermitian constraints *******************\n');

    A_hc.r = zeros(mr+2*mc,n2);
    A_hc.r(1:mr,:) = A.r;
    k1 = mr;
    k2 = mr+mc;
    for i = 1:mc
      M = mat(A.c(i,:));	% M contains the conjugate of the constraint matrix
      k1 = k1+1;
      A_hc.r(k1,:) = 0.5*vec(M+M');
      k2 = k2+1;
      A_hc.r(k2,:) = -0.5i*vec(M-M');
    end
    A_hc.c = [];
    b_hc.r = [b.r;
              real(b.c);
              imag(b.c)];
    b_hc.c = [];
    y0_hc.r = [y0.r;
               real(y0.c);
               imag(y0.c)];
    y0_hc.c = [];

    [x,y,s,infos] = sdolab(A_hc,b_hc,c,x0,y0_hc,options);

    if infos.flag; return; end

    if exist('tpi')
      tot_hc = infos.timer(6);
      tpi_hc = tot_hc/infos.iter;
      fprintf(fout,'\nRatio CPU time per iteration wrt sdolab in complex numbers    = %12.5e',tpi_hc/tpi);
      fprintf(fout,'\nRatio CPU time wrt sdolab in complex numbers                  = %12.5e',tot_hc/tot);
    end

    % get the complex solution and compare it with the one of sdolab in complex numbers

    if solver.sdolab04_in_complex

      X_hc = mat(x);
      fprintf(fout,'\nRelative error on X wrt the complex solution of sdolab_0.4    = %12.5e',norm(vec(X_hc-X))/norm(vec(X)));

      y_hc = [y.r;real(y.c);imag(y.c)];
      fprintf(fout,'\nRelative error on y wrt the complex solution of sdolab_0.4    = %12.5e',norm(y_hc-yv)/norm(yv));

      S_hc = mat(s);
      fprintf(fout,'\nRelative error on S wrt the complex solution of sdolab_0.4    = %12.5e\n',norm(vec(S_hc-S))/norm(vec(S)));

    end

  end

%-----------------------------------------------------------------------
% Transform the problem into real numbers
%-----------------------------------------------------------------------

  if ~solver.sdolab04_in_real & ~solver.sedumi; return; end

  fprintf(fout,'\n*** Tranformation of the problem into real numbers *************************\n');

  sqrt2 = sqrt(2);

  % Make the real constraints AA*xx = bb. If the k-th constraint is <Ak,X> = ak, the k-th row of A contains con(Ak);
  % indeed <A,X> = tr A'*X = sum_{ij} (A.')_{ij}*X_{ij}).

  AA = zeros(mr+2*mc,(2*n)^2);
  for k = 1:mr
    M = mat(conj(A.r(k,:))/sqrt2);	% the rows of A contain the conjugate of the constraint matrices
    Mr = real(M);
    Mi = imag(M);
    AA(k,:) = vec([Mr  -Mi
                   Mi   Mr]);		% the rows of AA contain the constraint real matrices
  end
  k1 = mr;
  k2 = mr+mc;
  for k = 1:mc
    M = mat(conj(A.c(k,:))/sqrt2);	% the rows of A contain the conjugate of the constraint matrices
    MH = 0.5*(M+M');
    MZ = 0.5i*(M-M');
    Mr = real(MH);
    Mi = imag(MH);
    k1 = k1+1;
    AA(k1,:) = vec([Mr  -Mi
                    Mi   Mr]);		% the rows of AA contain the constraint real matrices
    Mr = real(MZ);
    Mi = imag(MZ);
    k2 = k2+1;
    AA(k2,:) = vec([Mr  -Mi
                    Mi   Mr]);		% the rows of AA contain the constraint real matrices
  end

  bb = zeros(mr+2*mc,1);
  bb(1:mr)            = b.r;
  if mc
    bb(mr+1:mr+mc)      = real(b.c);
    bb(mr+mc+1:mr+2*mc) = imag(b.c);
  end

  % Make the real objectif cc'*xx

  C = mat(c/sqrt2);			% c contains the conjugate of C
  Cr = real(C);
  Ci = imag(C);
  cc = vec([Cr  -Ci
            Ci   Cr]);

  % Make the initial points xx0 and yy0

  X0 = mat(x0/sqrt2);
  X0r = real(X0);
  X0i = imag(X0);
  xx0 = vec([X0r  -X0i
             X0i   X0r]);

  if isempty(y0)
    yy0 = [];
  else
    if mr == 0
      yy0 = [real(y0.c); imag(y0.c)];
    else
      if mc == 0
        yy0 = y0.r;
      else
        yy0 = [y0.r; real(y0.c); imag(y0.c)];
      end
    end
  end

%-----------------------------------------------------------------------
% Run sdolab v-0.4 on the real number problem
%-----------------------------------------------------------------------

  % Put the data in the correct form for sdolab

  AAA.r  = AA;
  bbb.r  = bb;
  if isempty(yy0)
    yyy0 = [];
  else
    yyy0.r = yy0;
  end

  % Call sdolab

  ooptions               = [];
  ooptions.eps           = 0.5*options.eps;	% 1.e-10 for caseWB2, 1.e-09 for caseWB5 and case9
  ooptions.iter          = options.iter;
  ooptions.max_bigm_loop = options.max_bigm_loop;
  ooptions.cholesky      = options.cholesky;
  ooptions.max_stepsize  = options.max_stepsize;
  ooptions.neighborhood  = options.neighborhood;	%0.01;
  ooptions.fout          = options.fout;
  ooptions.verb          = options.verb;

  if solver.sdolab04_in_real

    fprintf(fout,'\n*** Problem in real numbers ************************************************\n');

    [xx,yy,ss,infos] = sdolab(AAA,bbb,cc,xx0,yyy0,ooptions);

    if exist('tpi')
      tot_r = infos.timer(6);
      tpi_r = tot_r/infos.iter;
      fprintf(fout,'\nRatio CPU time per iteration wrt sdolab in complex numbers    = %12.5e',tpi_r/tpi);
      fprintf(fout,'\nRatio CPU time wrt sdolab in complex numbers                  = %12.5e',tot_r/tot);
    end

    % get the complex solution and compare it with the one of sdolab in complex numbers

    if solver.sdolab04_in_complex

      XX   = mat(xx);
      XX11 = XX(1:n,1:n);
      XX12 = XX(1:n,n+1:nn);
      XX21 = XX(n+1:nn,1:n);
      XX22 = XX(n+1:nn,n+1:nn);
      XX_complex = (1/sqrt2)*(XX11+XX22) + (1i/sqrt2)*(XX21-XX12);
      fprintf(fout,'\nRelative error on X wrt the complex solution of sdolab_0.4    = %12.5e',norm(vec(XX_complex-X))/norm(vec(X)));

      fprintf(fout,'\nRelative error on y wrt the complex solution of sdolab_0.4    = %12.5e',norm(yy.r-yv)/norm(yv));

      SS   = mat(ss);
      SS11 = SS(1:n,1:n);
      SS12 = SS(1:n,n+1:nn);
      SS21 = SS(n+1:nn,1:n);
      SS22 = SS(n+1:nn,n+1:nn);
      SS_complex = (1/sqrt2)*(SS11+SS22) + (1i/sqrt2)*(SS21-SS12);
      fprintf(fout,'\nRelative error on S wrt the complex solution of sdolab_0.4    = %12.5e',norm(vec(SS_complex-S))/norm(vec(S)));

      % check whether xx and ss is in R(J)

      XX_ortho = 0.5*[XX11-XX22 XX12+XX21
                      XX12+XX21 XX22-XX11];
      fprintf(fout,'\nRelative norm of the projection of X on the othogonal to R(J) = %12.5e',norm(vec(XX_ortho))/norm(vec(XX)));

      SS_ortho = 0.5*[SS11-SS22 SS12+SS21
                      SS12+SS21 SS22-SS11];
      fprintf(fout,'\nRelative norm of the projection of S on the othogonal to R(J) = %12.5e\n',norm(vec(SS_ortho))/norm(vec(SS)));

    end

  end

%-----------------------------------------------------------------------
% Run sedumi on the real number problem
%-----------------------------------------------------------------------

  K.f = 0;	% nb of free variables
  K.l = 0;	% no nonnegative constraints
  K.q = 0;	% no Lorentz constraints
  K.s = 2*n;	% sizes of the psd constraints

  pars.fid  = fout;	% 0: SeDuMi runs quietly
  pars.free = 0;	% otherwise, SeDuMi sets pars.free=1 and does unclear things with the free variables
  pars.eps  = 0.5*options.eps;	% precision 'prec' of SeDuMi imposed to be the same as the one 'gap' of sdolab * 1/2

  if solver.sedumi

    fprintf(fout,'\n*** Problem in real numbers ************************************************\n\n');

    tic
    [xvec,y,infos] = sedumi(AA,bb,cc,K,pars);
    time = toc;

    fprintf(fout,'\nTiming for SeDuMi');
    fprintf(fout,'\n. total/iter  %13.6f sec',time/infos.iter);
    fprintf(fout,'\n. total       %13.6f sec\n',time);

    if exist('tpi')
      tpi_s = time/infos.iter;
      fprintf(fout,'\nRatio CPU time per iteration wrt sdolab in complex numbers    = %12.5e',tpi_s/tpi);
      fprintf(fout,'\nRatio CPU time wrt sdolab in complex numbers                  = %12.5e',time/tot);
    end

    % get the complex solution and compare it with the one of sdolab in complex numbers

    if solver.sdolab04_in_complex

      XX   = mat(xvec);
      XX11 = XX(1:n,1:n);
      XX12 = XX(1:n,n+1:nn);
      XX21 = XX(n+1:nn,1:n);
      XX22 = XX(n+1:nn,n+1:nn);
      XX_complex = (1/sqrt2)*(XX11+XX22) + (1i/sqrt2)*(XX21-XX12);
      fprintf(fout,'\nRelative error on X wrt the complex solution of sdolab_0.4    = %12.5e',norm(vec(XX_complex-X))/norm(vec(X)));

      fprintf(fout,'\nRelative error on y wrt the complex solution of sdolab_0.4    = %12.5e',norm(y-yv)/norm(yv));

      SS   = mat(cc-AA'*y);
      SS11 = SS(1:n,1:n);
      SS12 = SS(1:n,n+1:nn);
      SS21 = SS(n+1:nn,1:n);
      SS22 = SS(n+1:nn,n+1:nn);
      SS_complex = (1/sqrt2)*(SS11+SS22) + (1i/sqrt2)*(SS21-SS12);
      fprintf(fout,'\nRelative error on S wrt the complex solution of sdolab_0.4    = %12.5e',norm(vec(SS_complex-S))/norm(vec(S)));

      % check whether xx and ss is in R(J)

      XX_ortho = 0.5*[XX11-XX22 XX12+XX21
                      XX12+XX21 XX22-XX11];
      fprintf(fout,'\nRelative norm of the projection of X on the othogonal to R(J) = %12.5e',norm(vec(XX_ortho))/norm(vec(XX)));

      SS_ortho = 0.5*[SS11-SS22 SS12+SS21
                      SS12+SS21 SS22-SS11];
      fprintf(fout,'\nRelative norm of the projection of S on the othogonal to R(J) = %12.5e\n',norm(vec(SS_ortho))/norm(vec(SS)));

    end

  end

  return
